@@ -34,7 +34,6 @@ end of this file.
 #define __USE_MISC
 #include <ctype.h>
 #include <dirent.h>
-#include <errno.h>
 #include <getopt.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -71,7 +70,7 @@ typedef struct meminfo {
 #define PROCESS_HUGE_INDEX    0
 #define PROCESS_PRIVATE_INDEX 3
 
-meminfo_t process_meminfo[] = {
+static meminfo_t process_meminfo[] = {
         { PROCESS_HUGE_INDEX,  "huge", "Huge" },
         {        1,            "heap", "Heap" },
         {        2,            "stack", "Stack" },
@@ -80,7 +79,7 @@ meminfo_t process_meminfo[] = {
 
 #define PROCESS_MEMINFO_ROWS (sizeof(process_meminfo) / sizeof(process_meminfo[0]))
 
-meminfo_t numastat_meminfo[] = {
+static meminfo_t numastat_meminfo[] = {
         { 0, "numa_hit", "Numa_Hit" },
         { 1, "numa_miss", "Numa_Miss" },
         { 2, "numa_foreign", "Numa_Foreign" },
@@ -91,7 +90,7 @@ meminfo_t numastat_meminfo[] = {
 
 #define NUMASTAT_MEMINFO_ROWS (sizeof(numastat_meminfo) / sizeof(numastat_meminfo[0]))
 
-meminfo_t system_meminfo[] = {
+static meminfo_t system_meminfo[] = {
         {  0, "MemTotal", "MemTotal" },
         {  1, "MemFree", "MemFree" },
         {  2, "MemUsed", "MemUsed" },
@@ -139,19 +138,19 @@ meminfo_t system_meminfo[] = {
 // hash algorithms depend on having some unused buckets.
 
 #define HASH_TABLE_SIZE 151
-int hash_collisions = 0;
+static int hash_collisions = 0;
 
 struct hash_entry {
         char *name;
         int index;
 } hash_table[HASH_TABLE_SIZE];
 
-void init_hash_table(void)
+static void init_hash_table(void)
 {
         memset(hash_table, 0, sizeof(hash_table));
 }
 
-int hash_ix(char *s)
+static int hash_ix(char *s)
 {
         unsigned int h = 17;
         while (*s) {
@@ -161,7 +160,7 @@ int hash_ix(char *s)
         return (h % HASH_TABLE_SIZE);
 }
 
-int hash_lookup(char *s)
+static int hash_lookup(char *s)
 {
         int ix = hash_ix(s);
         while (hash_table[ix].name) {	// Assumes big table with blank entries
@@ -176,7 +175,7 @@ int hash_lookup(char *s)
         return -1;
 }
 
-int hash_insert(char *s, int i)
+static int hash_insert(char *s, int i)
 {
         int ix = hash_ix(s);
         while (hash_table[ix].name) {	// assumes no duplicate entries
@@ -250,43 +249,43 @@ typedef struct vtab {
 
 #define USUAL_GUTTER_WIDTH 1
 
-void set_row_flag(vtab_p table, int row, int flag)
+static inline void set_row_flag(vtab_p table, int row, int flag)
 {
         table->row_flags[row] |= (uint8_t)flag;
 }
 
-void set_col_flag(vtab_p table, int col, int flag)
+static inline void set_col_flag(vtab_p table, int col, int flag)
 {
         table->col_flags[col] |= (uint8_t)flag;
 }
 
-void clear_row_flag(vtab_p table, int row, int flag)
+static inline void clear_row_flag(vtab_p table, int row, int flag)
 {
         table->row_flags[row] &= (uint8_t)~flag;
 }
 
-void clear_col_flag(vtab_p table, int col, int flag)
+static inline void clear_col_flag(vtab_p table, int col, int flag)
 {
         table->col_flags[col] &= (uint8_t)~flag;
 }
 
-int test_row_flag(vtab_p table, int row, int flag)
+static inline int test_row_flag(vtab_p table, int row, int flag)
 {
         return ((table->row_flags[row] & (uint8_t)flag) != 0);
 }
 
-int test_col_flag(vtab_p table, int col, int flag)
+static inline int test_col_flag(vtab_p table, int col, int flag)
 {
         return ((table->col_flags[col] & (uint8_t)flag) != 0);
 }
 
-void set_col_justification(vtab_p table, int col, int justify)
+static inline void set_col_justification(vtab_p table, int col, int justify)
 {
         table->col_flags[col] &= (uint8_t)~COL_JUSTIFY_MASK;
         table->col_flags[col] |= (uint8_t)(justify & COL_JUSTIFY_MASK);
 }
 
-void set_col_width(vtab_p table, int col, uint8_t width)
+static inline void set_col_width(vtab_p table, int col, uint8_t width)
 {
         if (width >= SMALL_BUF_SIZE) {
                 width = SMALL_BUF_SIZE - 1;
@@ -294,78 +293,78 @@ void set_col_width(vtab_p table, int col, uint8_t width)
         table->col_width[col] = width;
 }
 
-void set_col_decimal_places(vtab_p table, int col, uint8_t places)
+static inline void set_col_decimal_places(vtab_p table, int col, uint8_t places)
 {
         table->col_decimal_places[col] = places;
 }
 
-void set_cell_flag(vtab_p table, int row, int col, int flag)
+static inline void set_cell_flag(vtab_p table, int row, int col, int flag)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->flags |= (uint32_t)flag;
 }
 
-void clear_cell_flag(vtab_p table, int row, int col, int flag)
+static inline void clear_cell_flag(vtab_p table, int row, int col, int flag)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->flags &= (uint32_t)~flag;
 }
 
-int test_cell_flag(vtab_p table, int row, int col, int flag)
+static inline int test_cell_flag(vtab_p table, int row, int col, int flag)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         return ((c_ptr->flags & (uint32_t)flag) != 0);
 }
 
-void string_assign(vtab_p table, int row, int col, char *s)
+static inline void string_assign(vtab_p table, int row, int col, char *s)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->type = CELL_TYPE_STRING;
         c_ptr->s = s;
 }
 
-void repchar_assign(vtab_p table, int row, int col, char c)
+static inline void repchar_assign(vtab_p table, int row, int col, char c)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->type = CELL_TYPE_REPCHAR;
         c_ptr->c[0] = c;
 }
 
-void double_assign(vtab_p table, int row, int col, double d)
+static inline void double_assign(vtab_p table, int row, int col, double d)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->type = CELL_TYPE_DOUBLE;
         c_ptr->d = d;
 }
 
-void long_assign(vtab_p table, int row, int col, int64_t l)
+static inline void long_assign(vtab_p table, int row, int col, int64_t l)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->type = CELL_TYPE_LONG;
         c_ptr->l = l;
 }
 
-void double_addto(vtab_p table, int row, int col, double d)
+static inline void double_addto(vtab_p table, int row, int col, double d)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->type = CELL_TYPE_DOUBLE;
         c_ptr->d += d;
 }
 
-void long_addto(vtab_p table, int row, int col, int64_t l)
+static inline void long_addto(vtab_p table, int row, int col, int64_t l)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         c_ptr->type = CELL_TYPE_LONG;
         c_ptr->l += l;
 }
 
-void clear_assign(vtab_p table, int row, int col)
+static inline void clear_assign(vtab_p table, int row, int col)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         memset(c_ptr, 0, sizeof(cell_t));
 }
 
-void zero_table_data(vtab_p table, int type)
+static void zero_table_data(vtab_p table, int type)
 {
         // Sets data area of table to zeros of specified type
         for (int row = table->header_rows; (row < ALL_TABLE_ROWS); row++) {
@@ -377,7 +376,7 @@ void zero_table_data(vtab_p table, int type)
         }
 }
 
-void sort_rows_descending_by_col(vtab_p table, int start_row, int stop_row, int col)
+static void sort_rows_descending_by_col(vtab_p table, int start_row, int stop_row, int col)
 {
         // Rearrange row_ix_map[] indices so the rows will be in
         // descending order by the value in the specified column
@@ -399,12 +398,7 @@ void sort_rows_descending_by_col(vtab_p table, int start_row, int stop_row, int
         }
 }
 
-void span(vtab_p table, int first_row, int first_col, int last_row, int last_col)
-{
-        // FIXME: implement row / col spannnig someday?
-}
-
-void init_table(vtab_p table, int header_rows, int header_cols, int data_rows, int data_cols)
+static void init_table(vtab_p table, int header_rows, int header_cols, int data_rows, int data_cols)
 {
         // init table sizes
         table->header_rows = header_rows;
@@ -463,7 +457,7 @@ void init_table(vtab_p table, int header_rows, int header_cols, int data_rows, i
         memset(table->col_decimal_places, 0, alloc_size);
 }
 
-void free_cell(vtab_p table, int row, int col)
+static void free_cell(vtab_p table, int row, int col)
 {
         cell_p c_ptr = GET_CELL_PTR(row, col);
         if ((c_ptr->type == CELL_TYPE_STRING)
@@ -474,7 +468,7 @@ void free_cell(vtab_p table, int row, int col)
         memset(c_ptr, 0, sizeof(cell_t));
 }
 
-void free_table(vtab_p table)
+static void free_table(vtab_p table)
 {
         if (table->cell != NULL) {
                 for (int row = 0; (row < ALL_TABLE_ROWS); row++) {
@@ -501,7 +495,7 @@ void free_table(vtab_p table)
         }
 }
 
-char *fmt_cell_data(cell_p c_ptr, int max_width, int decimal_places)
+static char *fmt_cell_data(cell_p c_ptr, int max_width, int decimal_places)
 {
         // Returns pointer to a static buffer, expecting caller to
         // immediately use or copy the contents before calling again.
@@ -536,7 +530,7 @@ char *fmt_cell_data(cell_p c_ptr, int max_width, int decimal_places)
         return buf;
 }
 
-void auto_set_col_width(vtab_p table, int col, int min_width, int max_width)
+static void auto_set_col_width(vtab_p table, int col, int min_width, int max_width)
 {
         int width = min_width;
         for (int row = 0; (row < ALL_TABLE_ROWS); row++) {
@@ -557,7 +551,7 @@ void auto_set_col_width(vtab_p table, int col, int min_width, int max_width)
         table->col_width[col] = (uint8_t)width;
 }
 
-void display_justified_cell(cell_p c_ptr, int row_flags, int col_flags, int width, int decimal_places)
+static void display_justified_cell(cell_p c_ptr, int row_flags, int col_flags, int width, int decimal_places)
 {
         char *p = fmt_cell_data(c_ptr, width, decimal_places);
         int l = strlen(p);
@@ -585,7 +579,7 @@ void display_justified_cell(cell_p c_ptr, int row_flags, int col_flags, int widt
         printf("%s", buf);
 }
 
-void display_table(vtab_p table,
+static void display_table(vtab_p table,
                    int screen_width,
                    int show_unseen_rows,
                    int show_unseen_cols,
@@ -693,29 +687,29 @@ void display_table(vtab_p table,
         }
 }
 
-int verbose = 0;
-int num_pids = 0;
-int num_nodes = 0;
-int screen_width = 0;
-int show_zero_data = 1;
-int compress_display = 0;
-int sort_table = 0;
-int sort_table_node = -1;
-int compatibility_mode = 0;
-int pid_array_max_pids = 0;
-int *pid_array = NULL;
-char *prog_name = NULL;
-double page_size_in_bytes = 0;
-double huge_page_size_in_bytes = 0;
+static int verbose = 0;
+static int num_pids = 0;
+static int num_nodes = 0;
+static int screen_width = 0;
+static int show_zero_data = 1;
+static int compress_display = 0;
+static int sort_table = 0;
+static int sort_table_node = -1;
+static int compatibility_mode = 0;
+static int pid_array_max_pids = 0;
+static int *pid_array = NULL;
+static char *prog_name = NULL;
+static double page_size_in_bytes = 0;
+static double huge_page_size_in_bytes = 0;
 
-void display_version_and_exit(void)
+static void display_version_and_exit(void)
 {
         char *version_string = "20130723";
         printf("%s version: %s: %s\n", prog_name, version_string, __DATE__);
         exit(EXIT_SUCCESS);
 }
 
-void display_usage_and_exit(void)
+static void display_usage_and_exit(void)
 {
         fprintf(stderr, "Usage: %s [-c] [-m] [-n] [-p <PID>|<pattern>] [-s[<node>]] [-v] [-V] [-z] [ <PID>|<pattern>... ]\n", prog_name);
         fprintf(stderr, "-c to minimize column widths\n");
@@ -729,7 +723,7 @@ void display_usage_and_exit(void)
         exit(EXIT_FAILURE);
 }
 
-int get_screen_width(void)
+static int get_screen_width(void)
 {
         int width = 80;
         char *p = getenv("NUMASTAT_WIDTH");
@@ -762,7 +756,7 @@ int get_screen_width(void)
         return width;
 }
 
-char *command_name_for_pid(int pid)
+static char *command_name_for_pid(int pid)
 {
         // Get the PID command name field from /proc/PID/status file.  Return
         // pointer to a static buffer, expecting caller to immediately copy result.
@@ -791,7 +785,7 @@ char *command_name_for_pid(int pid)
         return NULL;
 }
 
-void show_info_from_system_file(char *file, meminfo_p meminfo, int meminfo_rows, int tok_offset)
+static void show_info_from_system_file(char *file, meminfo_p meminfo, int meminfo_rows, int tok_offset)
 {
         // Setup and init table
         vtab_t table;
@@ -904,7 +898,7 @@ void show_info_from_system_file(char *file, meminfo_p meminfo, int meminfo_rows,
         free_table(&table);
 }
 
-void show_numastat_info(void)
+static void show_numastat_info(void)
 {
         if (!compatibility_mode) {
                 printf("\nPer-node numastat info (in MBs):\n");
@@ -912,13 +906,13 @@ void show_numastat_info(void)
         show_info_from_system_file("numastat", numastat_meminfo, NUMASTAT_MEMINFO_ROWS, 0);
 }
 
-void show_system_info(void)
+static void show_system_info(void)
 {
         printf("\nPer-node system memory usage (in MBs):\n");
         show_info_from_system_file("meminfo", system_meminfo, SYSTEM_MEMINFO_ROWS, 2);
 }
 
-void show_process_info(void)
+static void show_process_info(void)
 {
         vtab_t table;
         int header_rows = 2;
@@ -1115,7 +1109,7 @@ int node_and_digits(const struct dirent *dptr)
         return 1;
 }
 
-void init_node_ix_map_and_header(int compatibility_mode)
+static void init_node_ix_map_and_header(void)
 {
         // Count directory names of the form: /sys/devices/system/node/node<N>
         struct dirent **namelist;
@@ -1180,7 +1174,7 @@ void init_node_ix_map_and_header(int compatibility_mode)
         }
 }
 
-void free_node_ix_map_and_header(void)
+static void free_node_ix_map_and_header(void)
 {
         if (node_ix_map != NULL) {
                 free(node_ix_map);
@@ -1195,9 +1189,9 @@ void free_node_ix_map_and_header(void)
         }
 }
 
-double get_huge_page_size_in_bytes(void)
+static double get_huge_page_size_in_bytes(void)
 {
-        double huge_page_size = 0;;
+        double huge_page_size = 0;
         FILE *fs = fopen("/proc/meminfo", "r");
         if (!fs) {
                 perror("Can't open /proc/meminfo");
@@ -1218,7 +1212,7 @@ double get_huge_page_size_in_bytes(void)
         return huge_page_size * KILOBYTE;
 }
 
-int all_digits(char *p)
+static int all_digits(char *p)
 {
         if (p == NULL) {
                 return 0;
@@ -1229,12 +1223,12 @@ int all_digits(char *p)
         return 1;
 }
 
-int starts_with_digit(const struct dirent *dptr)
+static int starts_with_digit(const struct dirent *dptr)
 {
         return (isdigit(dptr->d_name[0]));
 }
 
-void add_pid_to_list(int pid)
+static void add_pid_to_list(int pid)
 {
         if (num_pids < pid_array_max_pids) {
                 pid_array[num_pids++] = pid;
@@ -1260,7 +1254,7 @@ int ascending(const void *p1, const void *p2)
         return *(int *)p1 - *(int *) p2;
 }
 
-void sort_pids_and_remove_duplicates(void)
+static void sort_pids_and_remove_duplicates(void)
 {
         if (num_pids > 1) {
                 qsort(pid_array, num_pids, sizeof(int), ascending);
@@ -1278,7 +1272,7 @@ void sort_pids_and_remove_duplicates(void)
         }
 }
 
-void add_pids_from_pattern_search(char *pattern)
+static void add_pids_from_pattern_search(char *pattern)
 {
         // Search all /proc/<PID>/cmdline files and /proc/<PID>/status:Name fields
         // for matching patterns.  Show the memory details for matching PIDs.
@@ -1407,7 +1401,7 @@ int main(int argc, char **argv)
         // with the old numastat perl script (which is included at the end of this
         // file for reference)
         compatibility_mode = (argc == 1);
-        init_node_ix_map_and_header(compatibility_mode);	// enumarate the NUMA nodes
+        init_node_ix_map_and_header();	// enumarate the NUMA nodes
         if (compatibility_mode) {
                 show_numastat_info();
                 free_node_ix_map_and_header();
@@ -1493,7 +1487,7 @@ foreach $nd (readdir(NODES)) {
 	next unless $nd =~ /node(\d+)/;
 	# On newer kernels, readdir may enumerate the 'node(\d+) subdirs
 	# in opposite order from older kernels--e.g., node{0,1,2,...}
-	# as opposed to node{N,N-1,N-2,...}.  Accomodate this by
+	# as opposed to node{N,N-1,N-2,...}.  Accommodate this by
 	# switching to new mode so that the stats get emitted in
 	# the same order.
         #print "readdir(NODES) returns $nd\n";
