@@ -6,13 +6,11 @@
 #include "util.h"
 #include "stream_lib.h"
 
-void usage(void)
+static void usage(void)
 {
 	exit(1);
 }
 
-char *policy = "default";
-
 /* Run STREAM with a numa policy */
 int main(int ac, char **av)
 {
@@ -21,9 +19,11 @@ int main(int ac, char **av)
 	long size;
 	int policy;
 
-	policy = parse_policy(av[1], av[2]); 
+	policy = parse_policy(av[1], av[2]);
+	if (policy == MPOL_MAX)
+		usage();
 
-        nodes = numa_allocate_nodemask();
+	nodes = numa_allocate_nodemask();
 
 	if (av[1] && av[2])
 		nodes = numa_parse_nodestring(av[2]);
@@ -31,13 +31,13 @@ int main(int ac, char **av)
 		printf ("<%s> is invalid\n", av[2]);
 		exit(1);
 	}
-	size = stream_memsize();  
+	size = stream_memsize();
 	map = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS,
-		   0, 0); 
-	if (map == (char*)-1) exit(1); 
+		   0, 0);
+	if (map == (char*)-1) exit(1);
 	if (mbind(map, size, policy, nodes->maskp, nodes->size, 0) < 0)
 		perror("mbind"), exit(1);
-	stream_init(map); 
+	stream_init(map);
 	stream_test(NULL);
 	return 0;
 }
